@@ -3,18 +3,15 @@
 # Licensed under the MIT license.
 #
 # backend_aml.py: support for running jobs under Azure Machine Learning
+
 import os
-import json
-import uuid
 import shutil
 import logging
 import urllib.request
 from hyperopt import hp
-from interface import implements
 
 from xtlib import utils
 from xtlib import errors
-from xtlib import scriptor
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import run_helper
@@ -26,10 +23,8 @@ from xtlib.hparams import hp_helper
 from xtlib.report_builder import ReportBuilder
 from xtlib.helpers.feedbackParts import feedback as fb
 from xtlib.helpers.notebook_builder import NotebookBuilder
-from xtlib.hparams.hp_client import DistWrapper, ListWrapper
 
 from .backend_base import BackendBase
-from .backend_interface import BackendInterface
 
 SECTION = "external-services"
 
@@ -654,6 +649,10 @@ class AzureML(BackendBase):
             ws = self.get_aml_ws(aml_ws_name)
             experiment = Experiment(ws, name=aml_exper_name)
 
+        is_distributed = args['distributed']
+        dist_training = args["distributed_training"]
+        dist_training = dist_training.lower()
+
         # debug code
         # targets = ComputeTarget.list(ws)
         # console.print("available compute targets in workspace {}: {}".format(ws, targets))
@@ -667,8 +666,10 @@ class AzureML(BackendBase):
             else:
                 if compute_target in ws.compute_targets:
                     # its the name of a known AML target for this ws (compute instance, cluster, etc.)
+                    # use the actual predefined compute_target object (not the string)
                     actual_target = ws.compute_targets[compute_target]
                 else:
+                    # it appears under certain conditions, we can just pass a string as the target
                     #errors.config_error("compute target '{}' does not exist in AML workspace '{}'".format(compute_target, aml_ws_name))
                     #errors.warning("compute target '{}' not defined in AML compute targets for AML workspace '{}'".format(compute_target, aml_ws_name))
                     actual_target = compute_target
@@ -714,10 +715,6 @@ class AzureML(BackendBase):
             framework = "none"
             framework_version = None
 
-        is_distributed = args['distributed']
-        dist_training = args["distributed_training"]
-        dist_training = dist_training.lower()
-
         import azureml
         from azureml.train.estimator import Estimator, Mpi, Gloo, Nccl
         from azureml.train.dnn import PyTorch, Chainer, TensorFlow
@@ -907,7 +904,7 @@ class AzureML(BackendBase):
         service_run_info = {}
         return service_run_info, service_info_by_node     
 
-    def submit_node_runs(self, job_id, node_runs, workspace, aml_ws_name, xt_exper_name, aml_exper_name, 
+    def submit_node_runs(self, job_id, node_runs, ws_name, aml_ws_name, xt_exper_name, aml_exper_name, 
         compute_def, resume_name, repeat_count, using_hp, compute, runs_by_box, code_dir, node_index, 
         show_aml_run_name, nodes, args):
 
@@ -956,7 +953,7 @@ class AzureML(BackendBase):
         # append job/run name to AML experiment name (make locating XT runs in AML portal easier)
         aml_exper_name += "__{}__{}".format(job_id, first_run_name)
 
-        estimator, experiment = self.create_estimator(job_id, workspace, aml_ws_name, xt_exper_name, aml_exper_name, first_run_name, 
+        estimator, experiment = self.create_estimator(job_id, ws_name, aml_ws_name, xt_exper_name, aml_exper_name, first_run_name, 
             code_dir, target_fn, arg_dict, compute_target, node_id, nodes, fake_submit, args)
 
         hp_config = args["hp_config"]
@@ -1017,7 +1014,7 @@ class AzureML(BackendBase):
             trainer = estimator
 
         run_name, monitor_cmd, aml_run_name, aml_run_number, aml_run_id = \
-            self.run_aml_job(job_id, workspace, aml_ws_name, trainer, experiment, xt_exper_name, aml_exper_name, compute_target, code_dir, first_run_name, 
+            self.run_aml_job(job_id, ws_name, aml_ws_name, trainer, experiment, xt_exper_name, aml_exper_name, compute_target, code_dir, first_run_name, 
                 box_name, node_index, repeat_count, fake_submit, args)
 
         if show_aml_run_name:
@@ -1030,7 +1027,7 @@ class AzureML(BackendBase):
             run_name = run["run_name"]
             run_names.append(run_name)
 
-        node_info = {"ws": workspace}
+        node_info = {"ws": ws_name}
 
         for run_name in run_names:
             # we only have 1 run, so OK to hold info in flat dict here
@@ -1126,7 +1123,7 @@ class AzureML(BackendBase):
 
     # API call
     def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
-        encoding='utf-8', use_best_log=True):
+        encoding='utf-8', use_best_log=True, log_source=None):
 
         FN_STDOUT_LOG = "azureml-logs/00_stdout.txt"
         FN_STDOUT2_LOG = "azureml-logs/70_driver_log.txt"
