@@ -1398,7 +1398,7 @@ class AzureBatch(BackendBase):
 
     # BACKEND API
     def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
-        encoding='utf-8', use_best_log=True, service_context=None):
+        encoding='utf-8', use_best_log=True, log_source=None, service_context=None):
         '''
         returns subset of specified log file as text.
         '''
@@ -1421,22 +1421,25 @@ class AzureBatch(BackendBase):
         file_path = os.path.basename(log_name)
 
         # try to first read from job storage (if task has completed)
-        node_index = utils.node_index(node_id)
+        if log_source != "live":
+            node_index = utils.node_index(node_id)
 
-        job_path = "nodes/node{}/after/service_logs/{}".format(node_index, file_path)
-        if self.store.does_job_file_exist(workspace, job_id, job_path):
-            new_text = self.store.read_job_file(workspace, job_id, job_path)
-            batch_status = "completed"
-            simple_status = "completed"
-            found_file = True
-
-        if not found_file:
-            job_path = "nodes/node{}/after/xt_logs/{}".format(node_index, file_path)
+            job_path = "nodes/node{}/after/service_logs/{}".format(node_index, file_path)
             if self.store.does_job_file_exist(workspace, job_id, job_path):
                 new_text = self.store.read_job_file(workspace, job_id, job_path)
                 batch_status = "completed"
                 simple_status = "completed"
                 found_file = True
+                log_source = "after_logs"
+
+            if not found_file:
+                job_path = "nodes/node{}/after/xt_logs/{}".format(node_index, file_path)
+                if self.store.does_job_file_exist(workspace, job_id, job_path):
+                    new_text = self.store.read_job_file(workspace, job_id, job_path)
+                    batch_status = "completed"
+                    simple_status = "completed"
+                    found_file = True
+                    log_source = "after_logs"
 
         if not found_file:
 
@@ -1469,13 +1472,15 @@ class AzureBatch(BackendBase):
                 new_bytes = output.getvalue()
                 new_text = new_bytes.decode(encoding)
                 next_offset = start_offset + len(new_bytes)
+                log_source = "live"
 
             except (batchmodels.BatchErrorException) as ex:
                 # interpret this error as task has terminated
                 console.diag("Exception reading batch log file: {}".format(ex))
 
         return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
-            "service_status": batch_status, "file_path": file_path, "service_context": {"task": task}, "found_file": found_file}
+            "service_status": batch_status, "file_path": file_path, "service_context": {"task": task}, "found_file": found_file, 
+            "log_source": log_source}
 
     # API call
     def get_simple_status(self, status):
