@@ -6,15 +6,19 @@
 # runnable standalone or from XT (xt plot 
 
 import os
-import argparse
-from os.path import exists
 import math
 import json
-import numpy as np
+import time
 import yaml
+import argparse
+import numpy as np
+from os.path import exists
+
+from yaml import nodes
 
 from xtlib import utils
 from xtlib import console
+from xtlib import constants
 from xtlib import search_helper
 from xtlib.hparams import hp_helper
 from xtlib.storage.store import Store
@@ -302,7 +306,8 @@ class Hyperparameter():
 
 class SearchHistory():
     ''' Loads, processes, and evaluates the runs of a single hyperparameter search job. '''
-    def __init__(self, args=None, agg_name=None, workspace=None, timeout=None, primary_metric=None, hp_config_file_name=None):
+    def __init__(self, args=None, agg_name=None, workspace=None, timeout=None, primary_metric=None, 
+        hp_config_file_name=None, max_workers=1, xt_config=None):
 
         global plt
         import matplotlib.pyplot as plt
@@ -318,6 +323,7 @@ class SearchHistory():
 
         self.args = args
         self.timeout = timeout
+        self.max_workers = max_workers
 
         self.DIR_PATH = "{}/{}/".format(args.job_dir, args.job)
         self.runs = []
@@ -327,13 +333,14 @@ class SearchHistory():
         self.xtstore = None
 
         self.ws_name = workspace if workspace else args.workspace
+        self.xt_config = xt_config
 
 
     def init_xtstore(self):
         if self.xtstore is None:
-            xt_config = get_merged_config(create_if_needed=False)
-            self.xt_config = xt_config
-            self.xtstore = Store(config=xt_config)
+            if self.xt_config is None:
+                self.xt_config = get_merged_config(create_if_needed=False)
+            self.xtstore = Store(config=self.xt_config)
 
     def load_hp_definitions(self):
         ''' Get the hyperparameter definitions, downloaded from Azure if necessary. '''
@@ -353,12 +360,20 @@ class SearchHistory():
 
     def load_runs(self):
         ''' Download the runs from Azure. '''
-        runs_file_name = "all_runs.jsonl"
+        runs_file_name = constants.ALL_RUNS_FN
         all_runs_file_path = self.DIR_PATH + runs_file_name
 
         if not self.args.no_download:
             self.init_xtstore()
-            self.xtstore.download_file_from_job(self.ws_name, self.args.job, runs_file_name, all_runs_file_path)
+
+            #console.print("  downloading all_runs.jsonl file...", end="", flush=True)
+            started = time.time()
+
+            nodes_read = self.xtstore.download_all_runs_blobs(self.ws_name, "job", self.args.job, 
+                all_runs_file_path, max_workers=self.max_workers)
+
+            elapsed = time.time() - started
+            console.print("{} nodes read (elapsed: {:.2f} secs)".format(nodes_read, elapsed))
 
         run_summaries = open(all_runs_file_path, 'r')
         self.num_incomplete_runs = 0
@@ -447,7 +462,11 @@ class SearchHistory():
         self.name_hparam_dict = {}
         self.hparams = []
         chosen_hp_value_dict = yaml.load(open(hp_config, 'r'), Loader=yaml.Loader)
-        hpname_valuelist_dict = chosen_hp_value_dict['hyperparameter-distributions']
+
+        # allow for older name "hparams"
+        hparams_name = "hparams" if "hparams" in chosen_hp_value_dict else "hyperparameter-distributions"
+        hpname_valuelist_dict = chosen_hp_value_dict[hparams_name]
+
         for hpname in hpname_valuelist_dict:
             valuelist = hpname_valuelist_dict[hpname]
 
@@ -503,6 +522,9 @@ class SearchHistory():
         self.load_hp_definitions()
         self.load_runs()
 
+        console.print("  analyzing runs...", end="", flush=True)
+        started = time.time()
+
         # Add the runs to a global runset, which will double as the sentinel of the runset list.
         global_runset = RunSet(None, None, None)
         for run in self.runs:
@@ -562,6 +584,9 @@ class SearchHistory():
                 ds[x] = d
                 x += 1
 
+        elapsed = time.time() - started
+        console.print(" (elapsed: {:.2f}".format(elapsed))
+
         from xtlib.report_builder import ReportBuilder   
         builder = ReportBuilder(self.xt_config, self.xtstore)
 
