@@ -476,7 +476,7 @@ def fake_call_from_job(job_name, workspace, fn_hp_search):
     from xtlib.hparams.hparam_search import HParamSearch
     hps = HParamSearch()
 
-    search_file_text = store.read_job_file(context.dest_name, context.hp_config)
+    search_file_text = store.read_job_file(context.ws, context.dest_name, context.hp_config)
     hps.generate_hparam_set("fn_config", search_file_text, "run999", [], store, context)
 
 if __name__ == '__main__':
