@@ -14,7 +14,10 @@ class RandomSearch(implements(HpSearchInterface)):
         pass
 
     def need_runs(self):
-        return False
+        # since this isn't normally used as a dynamic search algorithm,
+        # we use it for scale testing of MONGO logging and retrieval and
+        # return True here to enable that retrieval.
+        return True
 
     def search(self, run_name, store, context, hp_records, runs):
         '''
