@@ -46,6 +46,10 @@ from xtlib.report_builder import ReportBuilder
 from xtlib.helpers.feedbackProgress import FeedbackProgress
 
 class ImplStorageApi():
+    '''
+    API and implementation for XT storage-oriented commands.  Eventually, we would like to move
+    all implementation code from impl_storage.py into this module.  Also see comment at top of file.
+    '''
     def __init__(self, config, store):
         self.config = config
         self.store = store
@@ -76,11 +80,18 @@ class ImplStorageApi():
             errors.combo_error("import of archive files with multiple workspaces not yet supported")
 
         archive_version = contents["archive_version"]
-        if not archive_version in ["1"]:
+        is_v1_archive = archive_version == "1"
+
+        if not archive_version in ["1", "2"]:
             errors.general_error("unrecognized archive format: {}".format(archive_version))
 
-        workspace = workspaces[0]
-        jobs = contents["jobs"]
+        if is_v1_archive:
+            workspace = workspaces[0]
+            jobs = contents["jobs"]
+        else:
+            ws_info = workspaces[0]
+            workspace = ws_info["ws_name"]
+            jobs = ws_info["jobs"]
 
         if not new_workspace:
             new_workspace = workspace
@@ -109,25 +120,37 @@ class ImplStorageApi():
             else:
                 new_job_id = prev_job_id
 
+            # for v1 (global job names)
             if self.store.does_job_exist(new_workspace, new_job_id):
                 if not overwrite:
                     errors.user_error("cannot import to existing job name: {} (use --overwrite to overwrite)".\
                         format(new_job_id))
 
                 # for now, overwrite job "in place"
-                # TODO: delete job from MONGO and STORAGE here
+                # TODO: delete job from DATABASE and STORAGE here
                         
             runs = jc["runs"] 
 
             if show_output:
                 console.print("  importing: {} => {} ({} runs)".format(prev_job_id, new_job_id, len(runs)))
 
-            # create MONGO JOB document
-            mongo_job_fn = os.path.join(temp_dir, "mongo/jobs/{}/mongo_job.json".format(prev_job_id))
-            self.import_job_mongo_document(mongo_job_fn, new_workspace, prev_job_id, new_job_id)
+            # create DATABASE JOB document
+            if is_v1_archive:
+                db_name = "mongo"
+                jobs_path = "jobs"
+                runs_path = "workspaces/{}/runs".format(workspace)
+            else:
+                db_name = "database"
+                jobs_path = "workspaces/{}/jobs".format(workspace)
+                runs_path = "{}/{}/runs".format(jobs_path, prev_job_id)
+
+            database_job_fn = os.path.join(temp_dir, "{}/{}/{}/{}_job.json".format(db_name, jobs_path, prev_job_id, db_name))
+            assert os.path.exists(database_job_fn)
+
+            self.import_job_database_document(database_job_fn, new_workspace, prev_job_id, new_job_id)
 
             # create STORAGE JOB blobs
-            storage_job_path = os.path.join(temp_dir, "storage/jobs/{}".format(prev_job_id))
+            storage_job_path = os.path.join(temp_dir, "storage/{}/{}".format(jobs_path, prev_job_id))
             self.import_job_storage_blobs(storage_job_path, new_workspace, prev_job_id, new_job_id)
 
             # for each run in job
@@ -138,24 +161,25 @@ class ImplStorageApi():
                 run_number = run_helper.get_parent_run_number(run_name)
                 max_run_seen = max(max_run_seen, run_number)
 
-                # copy MONGO RUN document
-                mongo_run_fn = os.path.join(temp_dir, "mongo/workspaces/{}/runs/{}/mongo_run.json".format(workspace, run_name))
+                # copy DATABASE RUN document
+                database_run_fn = os.path.join(temp_dir, "{}/{}/{}/{}_run.json".format(db_name, runs_path, run_name, db_name))
+                assert os.path.exists(database_run_fn)
 
                 # this will return the new run name (if version 1 import)
-                new_run_name, end_id, next_child_num = self.import_run_mongo_document(archive_version, mongo_run_fn, workspace, new_workspace, prev_job_id,
+                new_run_name, end_id, next_child_num = self.import_run_database_document(archive_version, database_run_fn, workspace, new_workspace, prev_job_id,
                      new_job_id, run_name, next_child_num)
 
                 max_end_seen = max(max_end_seen, end_id)
                 
                 # copy STORAGE RUN blobs
-                storage_run_path = os.path.join(temp_dir, "storage/workspaces/{}/runs/{}".format(workspace, run_name))
+                storage_run_path = os.path.join(temp_dir, "storage/{}/{}".format(runs_path, run_name))
                 self.import_run_storage_blobs(storage_run_path, workspace, new_workspace, prev_job_id, new_job_id, new_run_name)
 
-        # update MONGO counters for new workspace
-        self.store.mongo.set_workspace_counters(new_workspace, 1+max_run_seen, 1+max_end_seen)
+        # update DATABASE counters for new workspace
+        self.store.database.set_workspace_counters(new_workspace, 1+max_run_seen, 1+max_end_seen)
 
-    def import_job_mongo_document(self, fn_mongo_job, new_workspace, prev_job_id, new_job_id):
-        text = file_utils.read_text_file(fn_mongo_job)
+    def import_job_database_document(self, fn_database_job, new_workspace, prev_job_id, new_job_id):
+        text = file_utils.read_text_file(fn_database_job)
         job = json.loads(text)
 
         # update job_id
@@ -166,8 +190,8 @@ class ImplStorageApi():
         # update workspace
         job["ws_name"] = new_workspace
 
-        # add to mongo
-        self.store.mongo.create_mongo_job(job)
+        # add to database
+        self.store.database.create_db_job(job)
 
     def import_job_storage_blobs(self, storage_job_path, new_workspace, prev_job_id, new_job_id):
 
@@ -178,8 +202,8 @@ class ImplStorageApi():
         self.upload(storage_job_path + "/**", ".", share=None, workspace=new_workspace, experiment=None, job=new_job_id, 
             run=None, feedback=False, show_output=False)
 
-    def import_run_mongo_document(self, archive_version, mongo_run_fn, workspace, new_workspace, prev_job_id, new_job_id, run_name, next_child_num):
-        text = file_utils.read_text_file(mongo_run_fn)
+    def import_run_database_document(self, archive_version, database_run_fn, workspace, new_workspace, prev_job_id, new_job_id, run_name, next_child_num):
+        text = file_utils.read_text_file(database_run_fn)
         run = json.loads(text)
 
         job_num = job_helper.get_job_number(new_job_id)
@@ -192,19 +216,20 @@ class ImplStorageApi():
 
         if store_utils.STORAGE_FORMAT == "1":
             run["ws"] = new_workspace
-            if "ws_name" in run:
-                del run["ws_name"]
+            utils.safe_delete(run, "ws_name")
         else:
+            # fixup v1 naming and duplicate "duration" prop
             run["ws_name"] = new_workspace
-            if "ws" in run:
-                del run["ws"]
+            utils.safe_delete(run, "ws")
+            utils.rename_dict_key(run, "node", "node_index")
+            utils.safe_delete(run, "duration")
 
         if run_name != new_run_name:
             run["run_name"] = new_run_name
             run["run_number"] = run_helper.get_run_number(new_run_name)
 
-        # add to mongo
-        self.store.mongo.create_mongo_run(run)
+        # add to database
+        self.store.database.create_db_run(run)
 
         end_id = utils.safe_value(run, "end_id")
 
@@ -220,7 +245,7 @@ class ImplStorageApi():
 
     # COMMAND
     def export_workspace(self, output_file, workspace, tags_all, tags_any, jobs, experiment, show_output=True, 
-        response=None):
+        response=None, force_v1=False):
 
         export = True
 
@@ -239,14 +264,15 @@ class ImplStorageApi():
             
             with tempfile.TemporaryDirectory(prefix="import-") as temp_dir:
                 self.export_workspace_core(temp_dir, output_file, workspace, tags_all, tags_any, jobs, experiment, 
-                    show_output=show_output)
+                    show_output=show_output, force_v1=force_v1)
 
             if show_output:
                 console.print("  export completed")
         else:
             console.print("  export cancelled")
 
-    def export_workspace_core(self, temp_dir, output_file, workspace, tags_all, tags_any, jobs, experiment, show_output):
+    def export_workspace_core(self, temp_dir, output_file, workspace, tags_all, tags_any, jobs, experiment, 
+        show_output, force_v1):
 
         # get specified jobs from workspace (by job name, or by workspace name)
         args = {"job_list": jobs, "tags_all": tags_all, "tags_any": tags_any, "workspace": workspace, "all": True,
@@ -259,19 +285,26 @@ class ImplStorageApi():
             console.print("\nexporting workspace {} ({} jobs) to: {}".format(workspace, len(job_list), output_file))
 
         # build a table of contents structure describing this archive
-        archive_version = "1"
+        use_v1 = force_v1 or (store_utils.STORAGE_FORMAT == "1")
+
+        archive_version = "1" if use_v1 else "2"
         build = constants.BUILD
         username = self.config.get("general", "username")
         dt = datetime.datetime.now()
         dt_text = str(dt)
         storage_name = self.store.get_name()
-        mongo_name = self.store.mongo.get_service_name()
+        database_name = self.store.database.get_service_name()
         
         workspaces = []
         jobs = []
+        db_name = "mongo" if use_v1 else "database"
+
         contents = {"user": username, "export_date": dt_text, "archive_version": archive_version, "xt_build": build, 
-            "storage_format": store_utils.STORAGE_FORMAT, "storage": storage_name, "mongo": mongo_name, 
-            "workspaces": workspaces, "jobs": jobs }
+            "storage_format": store_utils.STORAGE_FORMAT, "storage": storage_name, db_name: database_name, 
+            "workspaces": workspaces}
+
+        if use_v1:
+            contents["jobs"] = jobs
 
         first_job = None
         first_ws = None
@@ -281,44 +314,58 @@ class ImplStorageApi():
             job_id = jr["job"]
             job_ws = jr["workspace"] 
 
-            mongo_runs = self.store.mongo.get_info_for_runs(job_ws, {"job_id": job_id}, None, 
-                include_log_records=True)
-            run_names = [mr["run_name"] for mr in mongo_runs]
+            run_docs = self.store.database.get_info_for_runs(job_ws, {"job_id": job_id}, 
+                include_log_records=False)
+            run_names = [doc["run_name"] for doc in run_docs]
 
             if show_output:
-                console.print("  exporting: {} ({} runs)".format(job_id, len(mongo_runs)))
-
-            job_content = {"job_id": job_id, "workspace": job_ws, "runs": run_names}
-            jobs.append(job_content)
+                console.print("  exporting: {} ({} runs)".format(job_id, len(run_docs)))
 
             if first_job is None:
                 first_job = job_id
                 first_ws = job_ws
 
-                workspaces.append(job_ws)
+                if use_v1:
+                    ws_context = first_ws
+                else:
+                    ws_context = {"ws_name": first_ws, "jobs": jobs}
+
+                workspaces.append(ws_context)
+
+            job_content = {"job_id": job_id, "runs": run_names}
+            jobs.append(job_content)
 
             if job_ws != first_ws:
                 errors.combo_error("can only export jobs from a single workspace (job {} has ws={}, job {} as ws={})". \
                     format(first_job, first_ws, job_id, job_ws))
 
-            # copy MONGO JOB document
-            temp_mongo_path = os.path.join(temp_dir, "mongo/jobs/{}".format(job_id))
-            self.export_job_mongo_document(first_ws, job_id, temp_mongo_path)
+            if use_v1:
+                db_name = "mongo"
+                jobs_path = "jobs"
+                runs_path = "workspaces/{}/runs".format(first_ws)
+            else:
+                db_name = "database"
+                jobs_path = "workspaces/{}/jobs".format(first_ws)
+                runs_path = "{}/{}/runs".format(jobs_path, job_id)
+
+            # copy DATABASE JOB document
+            temp_database_path = os.path.join(temp_dir, "{}/{}/{}".format(db_name, jobs_path, job_id))
+            self.export_job_database_document(first_ws, job_id, temp_database_path, db_name)
 
             # copy STORAGE JOB blobs
-            temp_store_path = os.path.join(temp_dir, "storage/jobs/{}".format(job_id))
+            temp_store_path = os.path.join(temp_dir, "storage/{}/{}".format(jobs_path, job_id))
             self.export_job_storage_blobs(first_ws, job_id, temp_store_path)
 
             # for each run in job
-            for mr in mongo_runs:
+            for mr in run_docs:
 
-                # copy MONGO RUN document
+                # copy DATABASE RUN document
                 run_name = mr["run_name"]
-                temp_mongo_path = os.path.join(temp_dir, "mongo/workspaces/{}/runs/{}".format(job_ws, run_name))
-                self.export_run_mongo_document(mr, temp_mongo_path)
+                temp_database_path = os.path.join(temp_dir, "{}/{}/{}".format(db_name, runs_path, run_name))
+                self.export_run_database_document(mr, temp_database_path, db_name)
 
                 # copy STORAGE RUN blobs
-                temp_store_path = os.path.join(temp_dir, "storage/workspaces/{}/runs/{}".format(job_ws, run_name))
+                temp_store_path = os.path.join(temp_dir, "storage/{}/{}".format(runs_path, run_name))
                 self.export_run_storage_blobs(job_ws, job_id, run_name, temp_store_path)
 
         # add contents
@@ -331,16 +378,22 @@ class ImplStorageApi():
         prefix_len = 1 + len(temp_dir)
         file_helper.zip_up_filenames(output_file, filenames, compress=True, remove_prefix_len=prefix_len)
 
-    def export_job_mongo_document(self, ws_name, job_id, temp_mongo_path):
-        record = job_helper.get_job_record(self.store, ws_name, job_id)
+    def export_job_database_document(self, ws_name, job_id, temp_database_path, db_name):
+
+        # don't include log_records (no longer tracked by database)
+        fields_dict = {"job_info": 1, "job_stats": 1, "connect_info_by_node": 1, 
+            "service_info_by_node": 1, "secrets_by_node": 1, "runs_by_box": 1, 
+            "hparams": 1, "tags": 1}
+
+        record = job_helper.get_job_record(self.store, ws_name, job_id, fields_dict=fields_dict)
         text = json.dumps(record)
 
-        fn = os.path.join(temp_mongo_path, "mongo_job.json")
+        fn = os.path.join(temp_database_path, "{}_job.json".format(db_name))
         file_utils.write_text_file(fn, text)
 
-    def export_run_mongo_document(self, record, temp_mongo_path):
+    def export_run_database_document(self, record, temp_database_path, db_name):
         text = json.dumps(record)
-        fn = os.path.join(temp_mongo_path, "mongo_run.json")
+        fn = os.path.join(temp_database_path, "{}_run.json".format(db_name))
         file_utils.write_text_file(fn, text)
 
     def export_job_storage_blobs(self, first_ws, job_id, temp_store_path):
@@ -503,7 +556,7 @@ class ImplStorageApi():
 
             if len(file_names) == 0:
                 if show_output:
-                    console.print("no matching files found in: {}".format(what, actual_path))
+                    console.print("no matching {} found in: {}".format(what, actual_path))
                 return
             elif len(file_names) == 1:
                 what = "blob" if use_blobs else "file"
