@@ -161,63 +161,68 @@ def build_job_filter_dict(ws_name, job_list, user_to_actual, builder, workspace,
 
     return fd
 
-def get_job_property_dicts():
-    # user-friendly property names for jobs
-    user_to_actual = {"cluster": "pool_info.cluster", "docker": "pool_info.environment", "experiment": "exper_name",
-        "hold": "hold", "job": "job_id","job_num": "job_num", "low_pri": "pool_info.low-pri", 
-        "nodes": "pool_info.nodes", "primary_metric": "primary_metric", "queue": "pool_info.queue", 
-        "repeat": "repeat", "search": "search_type", "service_type": "pool_info.service", 
-        "sku": "pool_info.sku", "runs": "run_count", "search_style": "search_style", "started": "started", "target": "compute", "username": "username", 
-        "vc": "pool_info.vc", "vm_size": "pool_info.vm-size", "workspace": "ws_name",  
-        "hparams": "hparams", "tags": "tags", 
+user_to_actual = {"cluster": "pool_info.cluster", "docker": "pool_info.environment", "experiment": "exper_name",
+    "hold": "hold", "job": "job_id","job_num": "job_num", "low_pri": "pool_info.low-pri", 
+    "nodes": "pool_info.nodes", "primary_metric": "primary_metric", "queue": "pool_info.queue", 
+    "repeat": "repeat", "search": "search_type", "service_type": "pool_info.service", 
+    "sku": "pool_info.sku", "runs": "run_count", "search_style": "search_style", "started": "started", "target": "compute", "username": "username", 
+    "vc": "pool_info.vc", "vm_size": "pool_info.vm-size", "workspace": "ws_name",  
+    "hparams": "hparams", "tags": "tags", 
 
 
-        # dynamic properties (get updated at various stages of job)
-        "status": "job_status", "running_nodes": "running_nodes", 
-        "running_runs": "running_runs", "error_runs": "error_runs", "completed_runs": "completed_runs",
-        "total_runs": "total_runs", "db_retries": "db_retries", "storage_retries": "storage_retries",
-        "restarts": "restarts"
-        }
-
-    std_cols_desc = {
-        # the key here is the USER friendly name (not the physical name)
-        "cluster": "the Philly Cluster that the job ran on",
-        "docker": "the Docker environment specified for this job",
-        "experiment": "the experiment associated with the job",
-        "hold": "if the Azure Batch pool for this job was requested to be held open after runs were completed",
-        "job": "the job name",
-        "job_num": "the sort compatible, numeric portion of the job_id",
-        "low_pri": "if the nodes requested for this job were specified as preemptable",
-        "nodes": "the number of compute nodes requested for this job",
-        "primary_metric": "the name of the metric used for hyperparameter searching",
-        "queue": "the Philly queue that the job was submitted to",
-        "repeat": "the repeat count specified for the job",
-        "runs": "the total number of runs for the job",
-        "search": "the type of hyperparameter search requested",
-        "search_style": "describes how search is being accomplished (one of: static, dynamic, multi, repeat, single)",
-        "service_type": "the service type of the target",
-        "sku": "the machine type requested for this job",
-        "started": "the datetime when the job was started",
-        "target": "the name of the compute target the job was submitted to",
-        "username": "the login name of the user who submitted the job",
-        "vc": "the Philly Virtual Cluster that the job ran on",
-        "vm_size": "the Azure Batch machine size requested for this job",
-        "workspace": "the workspace associated with the job",
-
-        # dynamic properties (get updated at various stages of job)
-        "db_retries": "the total number of database retries performed by the runs of this node",
-        "status": "one of: submitted, running, completed",
-        "running_nodes": "the number of nodes in the running state",
-        "running_runs": "the number of this job's runs that are current running",
-        "completed_runs": "the number of runs that have completed (with or without errors)",
-        "error_runs": "the number of runs that have terminated with an error",
-        "storage_retries": "the total number of storage retries performed by the runs of this node",
-        "total_runs": "the total number of runs specified for the job",
-        "restarts": "the total number node restarts in this job",
+    # dynamic properties (get updated at various stages of job)
+    "status": "job_status", "running_nodes": "running_nodes", 
+    "running_runs": "running_runs", "error_runs": "error_runs", "completed_runs": "completed_runs",
+    "total_runs": "total_runs", "db_retries": "db_retries", "storage_retries": "storage_retries",
+    "restarts": "restarts"
     }
 
+std_cols_desc = {
+    # the key here is the USER friendly name (not the physical name)
+    "cluster": "the Philly Cluster that the job ran on",
+    "docker": "the Docker environment specified for this job",
+    "experiment": "the experiment associated with the job",
+    "hold": "if the Azure Batch pool for this job was requested to be held open after runs were completed",
+    "job": "the job name",
+    "job_num": "the sort compatible, numeric portion of the job_id",
+    "low_pri": "if the nodes requested for this job were specified as preemptable",
+    "nodes": "the number of compute nodes requested for this job",
+    "primary_metric": "the name of the metric used for hyperparameter searching",
+    "queue": "the Philly queue that the job was submitted to",
+    "repeat": "the repeat count specified for the job",
+    "runs": "the total number of runs for the job",
+    "search": "the type of hyperparameter search requested",
+    "search_style": "describes how search is being accomplished (one of: static, dynamic, multi, repeat, single)",
+    "service_type": "the service type of the target",
+    "sku": "the machine type requested for this job",
+    "started": "the datetime when the job was started",
+    "target": "the name of the compute target the job was submitted to",
+    "username": "the login name of the user who submitted the job",
+    "vc": "the Philly Virtual Cluster that the job ran on",
+    "vm_size": "the Azure Batch machine size requested for this job",
+    "workspace": "the workspace associated with the job",
+
+    # dynamic properties (get updated at various stages of job)
+    "db_retries": "the total number of database retries performed by the runs of this node",
+    "status": "one of: submitted, running, completed",
+    "running_nodes": "the number of nodes in the running state",
+    "running_runs": "the number of this job's runs that are current running",
+    "completed_runs": "the number of runs that have completed (with or without errors)",
+    "error_runs": "the number of runs that have terminated with an error",
+    "storage_retries": "the total number of storage retries performed by the runs of this node",
+    "total_runs": "the total number of runs specified for the job",
+    "restarts": "the total number node restarts in this job",
+}
+
+def get_job_property_dicts():
+    # user-friendly property names for jobs
     return user_to_actual, std_cols_desc
 
+def get_job_property_internal_name(name):
+    if name in user_to_actual:
+        return user_to_actual[name]
+    return None
+
 def get_list_jobs_records(store, config, args):
     list_nodes = utils.safe_value(args, "list_nodes")
 
