@@ -18,7 +18,7 @@ from xtlib import file_utils
 from xtlib import store_utils
 from xtlib import impl_storage
 
-from xtlib.storage.store import Store, store_from_context
+#from xtlib.storage.store import Store, store_from_context
 from xtlib.console import console
 from xtlib.impl_shared import StoreOnDemand
 from xtlib.helpers.xt_config import get_merged_config
@@ -120,8 +120,8 @@ class Run():
         run_cache_dir = None
         self.config = config
 
-        if config:
-            run_cache_dir = config.get("general", "run-cache-dir")
+        # if config:
+        #     run_cache_dir = config.get("general", "run-cache-dir")
 
         is_aml_run = bool(os.getenv("AML_WORKSPACE_NAME"))
         
@@ -278,6 +278,8 @@ class Run():
             # avoid calling close twice (once by app, once by auto_close)
             return
 
+        print("run.close() starting...")
+        
         # careful not to trigger store method if not needed so that local script runs
         # don't trigger need for authentication unnecessarily 
         has_store = False
