@@ -11,6 +11,8 @@ import time
 import yaml
 import uuid
 import shutil
+from typing import List
+from threading import Lock
 
 from xtlib.client import Client
 from xtlib.console import console
@@ -30,6 +32,7 @@ from xtlib import scriptor
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import job_helper
+from xtlib import store_utils
 from xtlib import process_utils
 from xtlib import box_information
 
@@ -99,7 +102,7 @@ class Runner():
         compute = args["target"]
         box_def = self.config.get("boxes", compute, suppress_warning=True)
 
-        compute_def = dict(self.config.get_compute_def(compute))
+        compute_def = dict(self.config.get_target_def(compute))
 
         if not "service" in compute_def:
             errors.config_error("compute target '{}' must define a 'service' property".format(compute))
@@ -163,7 +166,8 @@ class Runner():
 
     def build_first_run_for_node(self, node_index, run_count, total_run_count, box_name, run_script_path, parent_script_path, 
             using_hp, use_aml_hparam, run_specs, job_id, parent_name, cmds, compute_def, repeat_count, fake_submit, 
-            search_style, box_secret, args):
+            search_style, box_secret, node_count, run_name=None, args=None):
+            
         exper_name = args['experiment']
 
         box_info = box_information.BoxInfo(self.config, box_name, self.store, args=args)
@@ -197,15 +201,62 @@ class Runner():
         path = os.path.realpath(args["script"])
 
         run_name, full_run_name, box_name, pool = \
-            self.create_run(job_id, actual_parts, box_name=box_name, parent_name=parent_name, node_index=node_index, using_hp=using_hp, 
-                repeat=repeat_count, app_info=None, path=path, exper_name=exper_name, compute_def=compute_def, fake_submit=fake_submit, 
-                search_style=search_style, run_count=run_count, total_run_count=total_run_count, args=args)
+            self.create_run(job_id, actual_parts, box_name=box_name, parent_name=parent_name, 
+                node_index=node_index, using_hp=using_hp, 
+                repeat=repeat_count, app_info=None, path=path, exper_name=exper_name, compute_def=compute_def, 
+                fake_submit=fake_submit, search_style=search_style, run_count=run_count, 
+                total_run_count=total_run_count, run_name=run_name, args=args)
 
         run_data = {"run_name": run_name, "run_specs": run_specs, "box_name": box_name, "box_index": node_index, 
             "box_info": box_info, "repeat": repeat_count, "box_secret": box_secret}
 
         return run_data
        
+    def write_node_info_data(self, _id, ws_name, job_id, node_index, total_run_count, node_count, 
+        box_name, run_name, secret, service_info, exper_name, args):
+
+        target = args["target"]
+        str_now = utils.get_arrow_time_str()
+
+        job_num = job_helper.get_job_number(job_id)
+        node_num = job_num*1000*1000 + node_index
+        node_name = "{}/{}".format(job_id, node_index)
+
+        # compute node_run_count
+        first, last = self.core.compute_run_indexes_for_node(total_run_count, node_count, node_index)
+        node_run_count = last - first + 1
+        node_id = utils.node_id(node_index)
+        service_info_text = json.dumps(service_info)
+
+        dd = {"_id": _id, "ws_name": ws_name, "job_id": job_id, "exper_name": exper_name, 
+            "node_name": node_name, "node_id": node_id, "node_index": node_index, "node_num": node_num, 
+            "target": target, "total_runs": node_run_count, "box_name": box_name, "run_name": run_name,
+            "secret": secret, "service_info": service_info_text}
+
+        # write dd record to node_info
+        self.store.database.update_collection("node_info", ws_name, dd)
+
+    def write_node_stats_data(self, _id, ws_name):
+        str_now = utils.get_arrow_time_str()
+
+        dd = {"_id": _id, "node_status": "created", 
+            "completed_runs": 0, "error_runs": 0, "running_runs": 0,
+            "create_time": str_now, "restarts": 0, "db_retries": 0, "storage_retries": 0}
+
+        # write dd record to node_stats
+        self.store.database.update_collection("node_stats", ws_name, dd)
+
+    def write_node_tags_data(self, _id, ws_name, job_id, tag_dict):
+
+        if not tag_dict:
+            tag_dict = {}
+
+        tag_dict["_id"] = _id        
+        tag_dict["job_id"] = job_id        
+
+        # write dd record to node_stats
+        self.store.database.update_collection("node_tags", ws_name, tag_dict)
+
     def write_hparams_to_job(self, ws_name, job_id, cmds, fake_submit, using_hp, args):
         # write to job-level sweeps-list file
         #console.print("cmds=", cmds)   
@@ -231,7 +282,7 @@ class Runner():
 
     def create_run(self, job_id, user_cmd_parts, box_name="local", parent_name=None, rerun_name=None, node_index=0, 
             using_hp=False, repeat=None, app_info=None, path=None, exper_name=None, compute_def=None, fake_submit=False, 
-            search_style=None, run_count=None, total_run_count=None, args=None):
+            search_style=None, run_count=None, total_run_count=None, run_name=None, args=None):
         '''
         'create_run' does the following:
             - creates a new run name (from the job name and node)
@@ -246,7 +297,7 @@ class Runner():
         app_name = None   # app_info.app_name
         box_nane = args["box"]
         pool = args["pool"]
-        run_name = ""
+        
         log_to_store = self.config.get("logging", "log")
         aggregate_dest = args["aggregate_dest"]
 
@@ -276,13 +327,16 @@ class Runner():
             else:
                 is_parent = search_style != "single"
 
-                run_name = self.store.mongo.get_next_run_name(workspace, job_id, 
-                    is_parent, total_run_count, node_index)
+                if not run_name:
+                    run_name = self.store.database.get_next_run_name(workspace, job_id, 
+                        is_parent, total_run_count, node_index)
+
+                tag_dict = args["tags"]
 
                 self.store.start_run_core(workspace, run_name=run_name, exper_name=exper_name, box_name=box_name, app_name=app_name, 
                     username=username, repeat=repeat, pool=pool, job_id=job_id, node_index=node_index, sku=sku,
                     description=description, aggregate_dest=aggregate_dest, path=path, compute=compute, service_type=service_type, 
-                    search_style=search_style, is_parent=is_parent)
+                    search_style=search_style, is_parent=is_parent, tag_dict=tag_dict)
 
             console.diag("create_run: after start_run")
 
@@ -290,7 +344,7 @@ class Runner():
             xt_cmd = args["xt_cmd"]
 
             if not fake_submit:
-                self.store.log_run_event(workspace, run_name, "cmd", {"cmd": user_cmd_parts, "xt_cmd": xt_cmd })
+                self.store.log_run_event(workspace, run_name, "cmd", {"cmd": user_cmd_parts, "xt_cmd": xt_cmd }, job_id=job_id)
 
             # for now, don't log args (contain private credentials and not clear if we really need it)
             # record all "args" (from cmd line, user config, default config) in log (for audit/re-run purposes)
@@ -321,7 +375,7 @@ class Runner():
         # upload SWEEP file to job or experiment directory
         fn_sweeps = args["hp_config"]
         agg_dest = args["aggregate_dest"]
-        workspace = args["workspace"]
+        ws_name = args["workspace"]
 
         if not fn_sweeps:
             # must have extracted sweeps data from cmd line options
@@ -332,9 +386,9 @@ class Runner():
         target_name = file_utils.path_join(constants.HP_CONFIG_DIR, os.path.basename(fn_sweeps))
         
         if agg_dest == "experiment":
-            self.store.create_experiment_file(workspace, exper_name, target_name, sweeps_text)
+            self.store.create_experiment_file(ws_name, exper_name, target_name, sweeps_text)
         else:
-            self.store.create_job_file(workspace, job_id, target_name, sweeps_text)
+            self.store.create_job_file(ws_name, job_id, target_name, sweeps_text)
 
     def build_runs_by_box(self, job_runs, workspace):
         # build box_name => runs dict for job info file
@@ -419,7 +473,7 @@ class Runner():
         return script_dir
 
     def build_docker_cmd(self, docker_name:str, target: str, bootstrap_dir: str, job_secret: str, 
-            setup_name: str, args: [str]):
+            setup_name: str, args: List[str]):
         '''
         This is only called for backends that don't have their own support for running in docker.  Currently, this is
         just the POOL backend.  This function will replace the user's script cmd with:
@@ -609,11 +663,17 @@ class Runner():
         pip_packages = utils.safe_value(setup_def, "pip-packages")
         python_path = utils.safe_value(setup_def, "python-path")
         other_cmds = utils.safe_value(setup_def, "other-cmds")
+        pre_cmds = utils.safe_value(setup_def, "pre-cmds")
+        install_blobfuse = utils.safe_value(setup_def, "install-blobfuse")
 
         args["conda_packages"] = conda_packages if conda_packages else []
         args["pip_packages"] = pip_packages if pip_packages else []
         args["python_path"] = python_path if python_path else []
         args["other_cmds"] = other_cmds if other_cmds else []
+        args["pre_cmds"] = pre_cmds if pre_cmds else []
+
+        # install blobfuse defaults to True
+        args["install_blobfuse"] = True if install_blobfuse is None else install_blobfuse
 
         self.adjust_pip_packages(args)
 
@@ -691,6 +751,11 @@ class Runner():
             job_id = self.store.create_job(workspace)
         fb.feedback(job_id)
 
+        if experiment:
+            # create the experiment, if it doesn't already exist
+            if not self.store.does_experiment_exist(workspace, experiment):
+                self.store.create_experiment(workspace, experiment)
+
         # start the feedback (by parts)
         fb.feedback("{}: {}".format("target", compute))
 
@@ -714,41 +779,10 @@ class Runner():
         secrets_by_node = {}
         remote_control = args["remote_control"]
 
-        for i in range(run_count):
-            box_name = boxes[i]
-
-            # generate a box secret for talking to XT controller for this node
-            box_secret = str(uuid.uuid4()) if remote_control else ""
-
-            # build runs for box_name
-            run_data = self.build_first_run_for_node(i, run_count, total_run_count, 
-                boxes[i], target_file, ps_path, using_hp, using_aml_hparam, run_specs, job_id, 
-                parent_name, cmds, compute_def, repeat_count, fake_submit, search_style, box_secret, args)
-
-            # for now, adhere to the more general design of multiple runs per box
-            box_runs = [run_data]      
-            job_runs.append(box_runs)
-
-            node_id = utils.node_id(i)            
-            secrets_by_node[node_id] = box_secret
-
-            # FEEDBACK 
-            ptype = "single " if search_style == "single" else "parent "
-            if is_distributed:
-                ptype = "master "
-
-            if run_count == 1:
-                node_msg = "creating {}run".format(ptype)
-            else:
-                node_msg = "creating {}runs: {}/{}".format(ptype, i+1, run_count)
-
-            if service_type == "pool":
-                node_msg += ", box: " + box_name
-
-            fb.feedback(node_msg, id="node_msg")  # , add_seperator=is_last)
-            last_msg = node_msg
-
-            # run the job
+        self.build_all_node_runs(boxes, remote_control, run_count, total_run_count, 
+                target_file, ps_path, using_hp, using_aml_hparam, run_specs, job_id, 
+                parent_name, cmds, compute_def, repeat_count, fake_submit, search_style, job_runs, 
+                secrets_by_node, is_distributed, service_type, args)
 
         # build box: runs dict for job info file
         runs_by_box, last_run = self.build_runs_by_box(job_runs, workspace)
@@ -795,11 +829,13 @@ class Runner():
         code_zip = args["code_zip"]
     
         fb.feedback("uploading files: ", add_seperator=False)
+        copied_code_files = None
 
         if not fake_submit:
             if code_upload:
                 # upload users CODE FILES
-                self.core.upload_before_files_to_job(job_id, snapshot_dir, "before/code", code_omit, code_zip, "code", args)
+                copied_code_files = self.core.upload_before_files_to_job(job_id, snapshot_dir, "before/code", 
+                    code_omit, code_zip, "code", args)
 
             # upload DATA from data_local (do we need to keep this?  should we upload to normal DATA location, vs. job?)
             data_upload = args["data_upload"]
@@ -818,18 +854,32 @@ class Runner():
             - submitting the job (for each node, queue runs for that node)
             - return service job id (or list of them if per node)
         '''
+        code_uploaded_dir = snapshot_dir
+
+        # change the snapshot_dir to a new dir that contains only files that were uploaded
+        if not code_zip or code_zip == "none":
+            pass
+        elif copied_code_files:
+            code_uploaded_dir = snapshot_base + "/code_uploaded"
+            file_utils.ensure_dir_exists(code_uploaded_dir)
+            
+            for fn in copied_code_files:
+                fn_dst = code_uploaded_dir + "/" + os.path.basename(fn)
+                shutil.copyfile(fn, fn_dst)
 
         # SUBMIT JOB 
-        service_job_info, service_info_by_node = self.backend.submit_job(job_id, job_runs, workspace, compute_def, resume_name, 
-            repeat_count, using_hp, runs_by_box, experiment, snapshot_dir, adjustment_scripts, args)
+        service_job_info, service_info_by_node = \
+            self.backend.submit_job(job_id, job_runs, workspace, compute_def, resume_name, 
+                repeat_count, using_hp, runs_by_box, experiment, code_uploaded_dir, adjustment_scripts, args)
 
         # POST SUBMIT processing
         if not fake_submit:
 
             # mark runs as QUEUED
-            for runs in runs_by_box.values():
-                first_run = runs[0]
-                self.store.log_run_event(workspace, first_run["run_name"], "status-change", {"status": "queued"}) 
+            self.log_runs_queued(runs_by_box, workspace, job_id, total_run_count, secrets_by_node, 
+                service_info_by_node, args)
+
+            fb.feedback("logging job")
 
             # write the job info file (now that backend has had a chance to update it)
             job_num = int(job_id[3:])
@@ -842,13 +892,10 @@ class Runner():
             dynamic_runs_remaining = None if search_style == "single" else total_run_count
             node_count = len(runs_by_box)
 
-            # static_runs_by_node = None
-            # if schedule == "static":
-            #     static_runs_by_node = self.build_static_runs_by_node(total_run_count, node_count)
-            #console.diag("static_runs_by_node=", static_runs_by_node)
-
             # NOTE: for mongo v2, we only support schedule=static
-            child_runs_by_node, child_runs_key = self.store.mongo.build_child_runs_schedule_data(total_run_count, node_count)
+            #child_runs_by_node, child_runs_key = self.store.database.build_child_runs_schedule_data(total_run_count, node_count)
+
+            tag_dict = args["tags"]
 
             dd = {
                 # JOB_INFO: flat READONLY props (18)
@@ -882,10 +929,7 @@ class Runner():
                 "job_status": "submitted", 
                 "running_nodes": 0, 
                 "running_runs": 0, 
-                "started": utils.get_time(), 
-
-                # CHILD_RUNS
-                child_runs_key: child_runs_by_node,
+                "started": utils.get_arrow_time_str(), 
 
                 # SERVICE_INFO: embedded READONLY props BY NODE (2)
                 "runs_by_box": runs_by_box, 
@@ -897,10 +941,15 @@ class Runner():
 
                 # job-detectable HPARAMS
                 "hparams": job_hparams,
+
+                # TAGS
+                "tags": tag_dict,
             }
 
             self.store.log_job_info(workspace, job_id, dd)
 
+        fb.feedback("done", is_final=True)
+
         escape = args["escape"]
 
         # update lastrun/lastjob info
@@ -914,6 +963,157 @@ class Runner():
         # return values for API support (X)
         return cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, compute_def, job_id 
 
+    def log_run_and_node(self, workspace, box_name, runs_by_box, job_id, total_run_count, secrets_by_node, 
+        service_info_by_node, args):
+
+        runs_for_box = runs_by_box[box_name]
+        first_run = runs_for_box[0]
+        run_name = first_run["run_name"]
+        node_index = first_run["box_index"]
+        node_count = len(runs_by_box)
+        node_id = utils.node_id(node_index)
+
+        box_secret = secrets_by_node[node_id]
+        service_info = service_info_by_node[node_id]
+
+        ws_name = args["workspace"]
+        exper_name = args["experiment"]
+
+        id_for_node = "{}/{}/{}".format(ws_name, job_id, node_index)
+
+        # does format supports NODE INFO?
+        if store_utils.STORAGE_FORMAT == "2":
+            # create NODE_INFO record 
+            self.write_node_info_data(id_for_node, ws_name, job_id, node_index, total_run_count, node_count, 
+                box_name, run_name, box_secret, service_info, exper_name, args)
+
+            # create NODE_STATS record 
+            self.write_node_stats_data(id_for_node, ws_name)
+
+            # create NODE_TAGS record 
+            tag_list = args["tags"]
+            td = utils.tag_dict_from_list(tag_list)
+            self.write_node_tags_data(id_for_node, ws_name, job_id, td)
+
+        # log QUEUED event for flat/parent run        
+        self.store.log_run_event(workspace, run_name, "status-change", {"status": "queued"}) 
+
+    def log_runs_queued(self, runs_by_box, ws_name, job_id, total_run_count, secrets_by_node, 
+        service_info_by_node, args):
+
+        from threading import Lock
+        worker_lock = Lock()
+
+        # create each run on a worker thread
+        next_progress_num = 1
+        box_name_list = list(runs_by_box.keys())
+
+        def thread_worker(box_names, workspace):
+            for box_name in box_names:
+                nonlocal next_progress_num
+
+                self.log_run_and_node(workspace, box_name, runs_by_box, job_id, total_run_count, 
+                    secrets_by_node, service_info_by_node, args)
+
+                with worker_lock:
+                    node_msg = "logging runs: {}/{}".format(next_progress_num, len(box_name_list))
+                    fb.feedback(node_msg, id="log_msg")  # , add_seperator=is_last)
+                    next_progress_num += 1
+
+        max_workers = args["max_workers"]
+
+        # turn ON insert buffering
+        self.store.database.set_insert_buffering(100)
+
+        utils.run_on_threads(thread_worker, box_name_list, max_workers, [ws_name])
+
+        # turn OFF insert buffering
+        self.store.database.set_insert_buffering(0)
+
+        # change status of all runs for this job to "queued", in database
+        self.store.database.update_job_run_stats(ws_name, job_id, {"status": "queued"})
+
+        # TODO: change all parent run status from created to queued with single:
+        #   update [run_stats] set [status] = 'queued' where [_id] like 'ws4/job17_%'
+
+    def build_all_node_runs(self, boxes, remote_control, run_count, total_run_count, 
+        target_file, ps_path, using_hp, using_aml_hparam, run_specs, job_id, 
+        parent_name, cmds, compute_def, repeat_count, fake_submit, search_style, job_runs, 
+        secrets_by_node, is_distributed, service_type, args):
+
+        # create each run on a worker thread
+        thread_lock = Lock()   
+        next_progress_num = 1
+
+        def thread_worker(node_indexes, boxes, remote_control, run_count, total_run_count, 
+            target_file, ps_path, using_hp, using_aml_hparam, run_specs, job_id, 
+            parent_name, cmds, compute_def, repeat_count, fake_submit, search_style, job_runs, 
+            secrets_by_node, is_distributed, service_type, run_names, args):
+
+            node_count = len(node_indexes)
+
+            for node_index in node_indexes:
+                # generate a box secret for talking to XT controller for this node
+                box_secret = str(uuid.uuid4()) if remote_control else ""
+                run_name = run_names[node_index]
+
+                run_data = self.build_first_run_for_node(node_index, run_count, total_run_count, 
+                    boxes[node_index], target_file, ps_path, using_hp, using_aml_hparam, run_specs, job_id, 
+                    parent_name, cmds, compute_def, repeat_count, fake_submit, search_style, box_secret, 
+                    node_count, run_name=run_name, args=args)
+
+                with thread_lock:
+                    nonlocal next_progress_num
+
+                    # for now, adhere to the more general design of multiple runs per box (to be removed)
+                    box_runs = [run_data]      
+                    job_runs.append(box_runs)
+
+                    node_id = utils.node_id(node_index)            
+                    secrets_by_node[node_id] = box_secret
+
+                    # FEEDBACK 
+                    ptype = "single " if search_style == "single" else "parent "
+                    if is_distributed:
+                        ptype = "master "
+
+                    if run_count == 1:
+                        node_msg = "creating {}run".format(ptype)
+                    else:
+                        node_msg = "creating {}runs: {}/{}".format(ptype, next_progress_num, run_count)
+
+                    fb.feedback(node_msg, id="node_msg")  # , add_seperator=is_last)
+                    next_progress_num += 1
+
+        max_workers = args["max_workers"]
+        node_index_list = range(len(boxes))
+
+        # we want to create run names in "normal order" (not subject to random bg workers races)
+        # so, we create all the run_names up-front
+        ws_name = args["workspace"]
+        is_parent = search_style != "single"
+
+        run_names = []
+        for node_index in node_index_list:
+            run_name = self.store.database.get_next_run_name(ws_name, job_id, is_parent, total_run_count, node_index)
+            run_names.append(run_name)
+
+        args_for_worker = [boxes, remote_control, run_count, total_run_count, 
+            target_file, ps_path, using_hp, using_aml_hparam, run_specs, job_id, 
+            parent_name, cmds, compute_def, repeat_count, fake_submit, search_style, job_runs, 
+            secrets_by_node, is_distributed, service_type, run_names, args]
+
+        # turn ON insert buffering
+        self.store.database.set_insert_buffering(100)
+
+        utils.run_on_threads(thread_worker, node_index_list, max_workers, args_for_worker)
+
+        # turn OFF insert buffering (and flush buffers)
+        self.store.database.set_insert_buffering(0)
+
+        # sort job_runs to normal order
+        job_runs.sort(key=lambda r: r[0]["box_index"])
+
     def get_cmd_line_args_from_file(self, fn):
         with open(fn, "rt") as infile:
             text = infile.read()
@@ -983,6 +1183,10 @@ class Runner():
                     name, value = arg.split("=")
                     hparams[name] = value
                     pending_name = None
+
+                    if value.startswith("["):
+                        # in-line HPARAM SEARCH specification
+                        return None
                 else:
                     # for now, treat as option without value
                     hparams[arg] = None
@@ -997,7 +1201,11 @@ class Runner():
                     hparams[pending_name] = arg
                     pending_name = None
 
-        return hparams
+                    if arg.startswith("["):
+                        # in-line HPARAM SEARCH specification
+                        return None
+
+        return hparams if hparams else None
 
     def fixup_script_in_cmd(self, cmd):
         cmd_parts = utils.cmd_split(cmd)
@@ -1077,7 +1285,8 @@ class Runner():
 
         return (cmds, runsets) if cmds else None
 
-    def build_cmds_with_search(self, service_type, cmd_parts, parent_script, run_script, run_cmd_from_script, use_controller, dry_run, args):
+    def build_cmds_with_search(self, service_type, cmd_parts, parent_script, run_script, run_cmd_from_script, 
+        use_controller, dry_run, args):
         '''
         args:
             - service_type: the type of backend service being used (aml, batch, etc.)
@@ -1108,6 +1317,7 @@ class Runner():
         search_style = None
         cmds = None
         runsets = None
+        run_count = None
 
         # get run_cmd
         run_cmd = run_cmd_from_script
@@ -1135,11 +1345,14 @@ class Runner():
 
         if cmds:
             # STATIC HPARAM SEARCH
-            run_count = len(runsets) if runsets else len(cmds)
+            #run_count = len(runsets) if runsets else len(cmds)
+            run_count = len(cmds)
+            runsets = None      # do not process further in controller
             search_style = "static"
 
         runs = args["runs"]
         max_runs = args["max_runs"]
+        node_repeat = args["node_repeat"]
 
         # USER MULTI CMDS
         result = self.read_user_multi_commands(using_hp, run_script, cmd_parts, args)
@@ -1177,6 +1390,16 @@ class Runner():
             cmds = [new_run_cmd]
             show_run_report = False
 
+        if node_repeat:
+            # duplicate cmds so they are repeated N times on each node
+            # nodes are assigned runs in round-robin fashion
+            new_cmds = []
+            for cmd in cmds:
+                new_cmds += [cmd]*node_repeat
+            
+            cmds = new_cmds
+            run_count = len(cmds)
+
         if show_run_report:
             console.print()   
             dr = " (dry-run)" if dry_run else ""
@@ -1185,15 +1408,27 @@ class Runner():
 
             console.print("{} {}runs{}:".format(search_style, stype, dr))
 
-            if runsets:
-                console.print("  command: {}".format(cmds[0]))
-                console.print("  runsets:")
+            if node_repeat:
+                runsets = None
+                search_style = "static"
+                node_index = -1
 
-                for i, runset in enumerate(runsets):
-                    console.print("    {}. {}".format(i+1, runset))
-            else:
                 for i, run_cmd_parts in enumerate(cmds):
-                    console.print("  {}. {}".format(i+1, run_cmd_parts))
+                    if i % node_repeat == 0:
+                        node_index += 1
+
+                    node_id = utils.node_id(node_index)
+                    console.print("  {}. {} ({})".format(i+1, run_cmd_parts, node_id))
+            else:
+                if runsets:
+                    console.print("  command: {}".format(cmds[0]))
+                    console.print("  runsets:")
+
+                    for i, runset in enumerate(runsets):
+                        console.print("    {}. {}".format(i+1, runset))
+                else:
+                    for i, run_cmd_parts in enumerate(cmds):
+                        console.print("  {}. {}".format(i+1, run_cmd_parts))
 
             console.print()   
 
@@ -1210,8 +1445,8 @@ class Runner():
             - args: dictionary of XT run cmd args/options
 
         processing:
-            - gather hyperparameter search specs from either ML app options or
-              user-specified hp-config file
+            - gather hyperparameter search specs from either special app command line options or
+              user-specified hp-config file (.yaml)
             - if doing random or grid search, generate the commands that comprise
               the search
              
@@ -1228,8 +1463,8 @@ class Runner():
         fn_sweeps = args["hp_config"]
         static_search = args["static_search"]
 
-        if not cmd_line:
-            cmd_line = " ".join(cmd_parts)
+        # if not cmd_line:
+        #     cmd_line = " ".join(cmd_parts)
 
         # default return values
         run_cmd = cmd_line
@@ -1278,8 +1513,8 @@ class Runner():
 # flat functions
 def update_compute_def_from_cmd_options(compute_def, hold=False):
 
-    compute_props = ["vm_size", "azure_image", "low_pri", "box_class", "docker", "setup", "boxes", 
-    "vc", "cluster", "queue", "sku", "nodes"]
+    compute_props = ["vm-size", "azure-image", "low-pri", "box-class", "docker", "setup", "boxes", 
+        "vc", "cluster", "queue", "sku", "nodes"]
 
     # apply any compute-def properties found in explicit args to compute_def
     explict_opts = qfe.get_explicit_options()
