@@ -1173,6 +1173,14 @@ class MongoDB2(implements(DbInterface)):
 
         return job_completed
 
+    # API call
+    def set_job_status(self, ws_name, job_id, status):
+        if self.update_job_stats:
+            _id = self.make_key(ws_name, job_id)
+            dbx = self.get_db(ws_name)
+            cmd = lambda: dbx["job_stats"].find_and_modify( {"_id": _id}, update={"$set": {"job_status": status} })
+            result2 = self.mongo_with_retries("set_job_status", dbx, cmd)
+        
     def update_run_at_end(self, ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics):
         # update run document on Mongo DB
         dbx = self.get_db(ws_name)
