@@ -8,6 +8,11 @@ import re
 import requests
 import logging
 from interface import implements
+
+# # debug
+# import azure.storage.blob as bb
+# print("azure blob storage version: ", bb.__version__)
+
 from azure.storage.blob import AppendBlobService, BlockBlobService, PublicAccess
 
 from xtlib import utils
@@ -56,7 +61,7 @@ class AzureBlobStore21(implements(StoreInterface)):
         self.append_bs = AppendBlobService(account_name=self.storage_id, account_key=self.storage_key,
             request_session=sess2) 
 
-        print("created azure blob services with {} pool connections".format(self.pool_connections))
+        #print("created azure blob services with {} pool connections".format(self.pool_connections))
 
         self.set_retries(self.max_retries)
 
