@@ -34,9 +34,10 @@ class StoreBlobObjs():
     suppress the STDERR output from Azures API's when they have an _error to keep our
     console output a bit cleaner (easier to follow what's happening).
     '''
-    def __init__(self, storage_creds, provider_code_path, max_retries=25):
+    def __init__(self, storage_creds, provider_code_path, max_retries=25, owner=None):
 
         self.provider_name = storage_creds["provider"]
+        self.owner = owner
 
         # instantiate the provider
         class_ctr = utils.get_class_ctr(provider_code_path)
@@ -52,12 +53,12 @@ class StoreBlobObjs():
         storage_name = self.provider.get_service_name()
         return storage_name
         
-    def validate_storage_and_mongo(self, mongo):
+    def validate_storage_and_db(self, db):
         '''
         1. ensure storage has been initialized for XT
-        2. ensure mongo and storage point to each other
+        2. ensure database and storage point to each other
         3. update storage format if needed
-        4. update mongo format if needed
+        4. update database format if needed
         '''
 
         # ensure storage has been initialized for XT
@@ -65,38 +66,38 @@ class StoreBlobObjs():
 
         # ensure mongo points to our storage
         storage_name = self.provider.get_service_name()
-        connected_mongo = mongo.get_service_name()
+        connected_db = db.get_service_name()
 
-        mongo_info = mongo.get_mongo_info()
-        paired_storage = utils.safe_value(mongo_info, "paired_storage")
+        db_info = db.get_db_info()
+        paired_storage = utils.safe_value(db_info, "paired_storage")
         if paired_storage and storage_name != paired_storage:
-            errors.combo_error("mongo paired with storage service='{}', but passed XT storage service='{}'".format(  \
+            errors.combo_error("db paired with storage service='{}', but passed XT storage service='{}'".format(  \
                 paired_storage, storage_name))
 
         storage_info = self._get_storage_info()
-        paired_mongo = utils.safe_value(storage_info, "paired_mongo")
-        if paired_mongo and connected_mongo != paired_mongo:
-            errors.combo_error("this storage paired with mongo service='{}', but passed connection string for mongo service='{}'".format(  \
-                connected_mongo, paired_mongo))
+        paired_db = utils.safe_value(storage_info, "paired_db")
+        if paired_db and connected_db != paired_db:
+            errors.combo_error("this storage paired with db service='{}', but passed connection string for db service='{}'".format(  \
+                connected_db, paired_db))
 
         if storage_info["storage_format"] != constants.STORAGE_FORMAT_V2:
             errors.store_error("XT storage format {} expected but found: {}".format(constants.STORAGE_FORMAT_V2, storage_info["storage_format"]))
 
         if not paired_storage:
-            mongo_info = {"paired_storage": storage_name, "storage_format": constants.STORAGE_FORMAT_V2}
-            mongo.set_mongo_info(mongo_info)
+            db_info = {"paired_storage": storage_name, "storage_format": constants.STORAGE_FORMAT_V2}
+            db.set_db_info(db_info)
 
-        if not paired_mongo:
-            storage_info = {"paired_mongo": connected_mongo, "storage_format": constants.STORAGE_FORMAT_V2}
+        if not paired_db:
+            storage_info = {"paired_db": connected_db, "storage_format": constants.STORAGE_FORMAT_V2}
             self._set_storage_info(storage_info)
 
-        if mongo_info["storage_format"] != constants.STORAGE_FORMAT_V2:
-            errors.store_error("XT mongo format {} expected but found: {}".format(constants.STORAGE_FORMAT_V2, mongo_info["storage_format"]))
+        if db_info["storage_format"] != constants.STORAGE_FORMAT_V2:
+            errors.store_error("XT db format {} expected but found: {}".format(constants.STORAGE_FORMAT_V2, db_info["storage_format"]))
 
         # only check once, (takes .5 secs if already imported)
         # remove this check after all XT users have imported (approx. Dec 2019)
-        # but keep around (good for mongodb repair, if needed)
-        #self.import_jobs_to_mongo_if_needed(mongo)
+        # but keep around (good for dbdb repair, if needed)
+        #self.import_jobs_to_db_if_needed(db)
 
     def import_jobs_to_mongo_if_needed(self, mongo):
         console.diag("before mongo import check")
@@ -358,7 +359,7 @@ class StoreBlobObjs():
         #console.print("_copy_files: container=", container, ", ws_wildcard=", ws_wildcard, ", to_path=", to_path)
         names = self._list_wild_blobs(container, from_path, from_wc_target)
         #console.print("names=", names)
-        blob_dir = path
+        blob_dir = to_path
         bd_index = 1 + len(blob_dir)   # add for for trailing slash
         #console.print("blob_dir=", blob_dir, ", bd_index=", bd_index)
 
@@ -652,7 +653,9 @@ class StoreBlobObjs():
         self._check_ws_name(ws_name)
 
         if self.does_workspace_exist(ws_name):
-            errors.store_error("workspace already exists: {}".format(ws_name))
+            #errors.store_error("workspace already exists: {}".format(ws_name))
+            # TEMP TEMP TEMP
+            return
 
         # note: this operation often must retry several times if same container has just been deleted
         #console.print("creating workspace=", ws_name)
@@ -889,28 +892,42 @@ class StoreBlobObjs():
     def run_files(self, ws_name, run_name, use_blobs=False, job_id=None):
         if use_blobs:
             if not job_id:
-                job_id = "job" + str(run_helper.get_parent_run_number(run_name))
+                # here, we support job_id and run_name NOT being tied together for import
+                # scenarios, where changing the job_id or run_name of imported records
+                # can create further issues.
+
+                #job_id = "job" + str(run_helper.get_parent_run_number(run_name))
+                _id = "{}/{}".format(ws_name, run_name)
+                record = self.owner.database.get_info_for_runs(ws_name, {"_id": _id}, {"job_id": 1})
+                job_id = utils.safe_cursor_value(record, "job_id")
+
+                # TODO: fix this so it doesn't break for imported workspaces with job != run names
+                #assert job_id
+                if not job_id:
+                    # this only works for job/run name sync. 
+                    job_id = "job{}".format(run_helper.get_parent_run_number(run_name))
+
             return RunBlobs(self, ws_name, job_id, run_name)
         else:
-            return store_azure_file.RunFiles(self, ws_name, run_name)
+            errors.general_error("store API only supports blob objects")
 
     def experiment_files(self, ws_name, exper_name, use_blobs=False):
         if use_blobs:
             return ExperimentBlobs(self, ws_name, exper_name)
         else:
-            return store_azure_file.ExperimentFiles(self, ws_name, exper_name)
+            errors.general_error("store API only supports blob objects")
 
     def workspace_files(self, ws_name, use_blobs=False):
         if use_blobs:
             return WorkspaceBlobs(self, ws_name)
         else:
-            return store_azure_file.WorkspaceFiles(self, ws_name)
+            errors.general_error("store API only supports blob objects")
 
     def job_files(self, ws_name, job_name, use_blobs=False):
         if use_blobs:
             return JobBlobs(self, ws_name, job_name)
         else:
-            return store_azure_file.JobFiles(self, ws_name, job_name)
+            errors.general_error("store API only supports blob objects")
 
     def root_files(self, root_name, use_blobs=False):
         if use_blobs:
