@@ -25,15 +25,18 @@ from xtlib.impl_shared import ImplShared
 logger = logging.getLogger(__name__)
 
 def get_fn_local_config(args):
+    '''
+    gets path to local config file (specified on cmd line or defaulting to ./xt_config.yaml)
+    '''
     # default value
     fn = os.path.join(".", constants.CONFIG_FN)
 
-    # is this a run cmd whose script is a .yaml file?
+    # is config file specified by --config option or as a run command with .yaml file script?
     cmd = " ".join(args)
     parts = utils.cmd_split(cmd)
     found_run = False
 
-    for part in parts:
+    for i, part in enumerate(parts):
         # don't change the order of these checks
         if found_run:
             if part.endswith(".yaml"):
@@ -47,15 +50,29 @@ def get_fn_local_config(args):
                         # only print if its not the current directory
                         console.print("using script config file: {}".format(fn))
             break
+        
+        if part.startswith("--"):
+            # process pre-cmd option
+            subs = part[2:].split("=", 1)
+            option = subs[0]
+            if qfe.match(option, "config"):
+                if len(subs) == 1:
+                    i += 1
+                    np = None
+                    if i < len(parts):
+                        np = parts[i]
+                else:
+                    np = subs[1]
+                if np:
+                    return np
 
-        if not part.startswith("--"):
+        else:
             if part == "run":
                 found_run = True
             else:
                 # some other command
                 break
 
-
     return fn
 
 def main(cmd=None, new_start_time=None, capture_output=False, basic_mode=False, raise_syntax_exception=True):
